@@ -215,7 +215,7 @@ exchange_oof_set (ExchangeAccount *account, gboolean oof, const char *message)
 	if (message) {
 		char *body, *message_enc;
 
-		message_enc = e2k_uri_encode (message, NULL);
+		message_enc = e2k_uri_encode (message, FALSE, NULL);
 		body = g_strdup_printf ("Cmd=options&OofState=%d&"
 					"OofReply=%s",
 					oof ? 1 : 0, message_enc);
